﻿using System;
using System.Windows;
using Microsoft.Xaml.Behaviors;

namespace TelerikRadGridViewProblems
{
	// Comes from https://pixytech.com/2014/03/12/attached-behaviors-memory-leaks/ (but modified to use WeakReference<T>)
	public abstract class BehaviorBase<T> : Behavior<T> where T : FrameworkElement
	{
		bool _isSetup;
		bool _isHookedUp;
		WeakReference<T> _weakTarget;

		protected virtual void OnSetup()
		{
		}

		protected virtual void OnCleanup()
		{
		}

		protected override void OnChanged()
		{
			if (this.AssociatedObject != null)
				this.HookupBehavior(this.AssociatedObject);
			else
				this.UnHookupBehavior();
		}

		void OnTarget_Loaded(object sender, RoutedEventArgs e) => this.SetupBehavior();

		void OnTarget_Unloaded(object sender, RoutedEventArgs e) => this.CleanupBehavior();

		void HookupBehavior(T target)
		{
			if (!this._isHookedUp)
			{
				this._weakTarget = new WeakReference<T>(target);
				this._isHookedUp = true;
				target.Unloaded += this.OnTarget_Unloaded;
				target.Loaded += this.OnTarget_Loaded;
				this.SetupBehavior();
			}
		}

		void UnHookupBehavior()
		{
			if (this._isHookedUp)
			{
				this._isHookedUp = false;
				var target = this.AssociatedObject ?? (this._weakTarget.TryGetTarget(out var t) ? t : null);
				if (target != null)
				{
					target.Unloaded -= this.OnTarget_Unloaded;
					target.Loaded -= this.OnTarget_Loaded;
				}
				this.CleanupBehavior();
			}
		}

		void SetupBehavior()
		{
			if (!this._isSetup)
			{
				this._isSetup = true;
				this.OnSetup();
			}
		}

		void CleanupBehavior()
		{
			if (this._isSetup)
			{
				this._isSetup = false;
				this.OnCleanup();
			}
		}
	}
}
