﻿using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using PostSharp.Patterns.Model;
using PostSharp.Patterns.Xaml;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GridView;

namespace TelerikRadGridViewProblems
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow() => this.InitializeComponent();
	}

	/// <summary>
	/// The view model for <see cref="MainWindow" />.
	/// </summary>
	[NotifyPropertyChanged]
	class MainWindowViewModel
	{
		public ItemViewModel CurrentItem { get; set; }

		// Generates an initial set of data
		public ObservableCollection<ItemViewModel> Items { get; } = new ObservableCollection<ItemViewModel>(Enumerable.Range(0, 26).Select(i =>
		{
			char c1 = (char)((int)'A' + i);
			char c2 = (char)((int)'A' + (25 - i));
			return new ItemViewModel
			{
				Id = $"{c1}{c2}",
				Name = $"{c1}{c1}{c2}{c2} {c2}{c2}{c1}{c1}",
				Active = i % 2 != 0
			};
		}));

		#region Commands

		#region GridViewLoaded

		[Command]
		public ICommand GridViewLoadedCommand { get; set; }

		public void ExecuteGridViewLoaded(RoutedEventArgs e)
		{
			var gridView = e.Source as RadGridView;
			gridView.KeyboardCommandProvider = new MyKeyboardCommandProvider(gridView);
			gridView.Items.CollectionChanged += (s, args) =>
			{
				if (args.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
				{
					this.CurrentItem = args.NewItems[0] as ItemViewModel;
					gridView.CancelEdit();
					gridView.CurrentCellInfo = new GridViewCellInfo(args.NewItems[0], gridView.Columns[0], gridView);
					gridView.ScrollIntoViewAsync(args.NewItems[0], gridView.Columns[0], null);
				}
			};
		}

		#endregion

		#region RadGridView events

		#region AddingNewDataItem

		[Command]
		public ICommand AddingNewDataItemCommand { get; set; }

		public void ExecuteAddingNewDataItem(GridViewAddingNewEventArgs e)
		{
			this.CurrentItem = null;
			var gridView = e.OwnerGridViewItemsControl as RadGridView;
			gridView.CurrentColumn = gridView.Columns[0]; // Force the first column to be the current column on a new row
		}

		#endregion

		#endregion

		#endregion
	}
}
