﻿using System;
using System.Reflection;
using System.Windows;
using PostSharp.Aspects;
using PostSharp.Extensibility;
using PostSharp.Serialization;

namespace TelerikRadGridViewProblems
{
	/// <summary>
	/// A PostSharp aspect that automatically makes a WPF routed event for a standard .NET event.
	/// </summary>
	[PSerializable]
	public sealed class RoutedEventAttribute : EventInterceptionAspect
	{
		Type DeclaringType { get; set; }

		Type EventHandlerType { get; set; }

		RoutedEvent RoutedEvent { get; set; }

		string TargetName { get; set; }

		public override void CompileTimeInitialize(EventInfo targetEvent, AspectInfo aspectInfo)
		{
			this.TargetName = targetEvent.Name;
			this.EventHandlerType = targetEvent.EventHandlerType;
			this.DeclaringType = targetEvent.DeclaringType;
		}

		public override bool CompileTimeValidate(EventInfo targetEvent)
		{
			if (!typeof(IInputElement).IsAssignableFrom(targetEvent.DeclaringType))
			{
				Message.Write(targetEvent, SeverityType.Error, "MY001",
					$"[RoutedEvent] cannot be applied to {targetEvent.Name} in class {targetEvent.DeclaringType.Name} as the class does not implement IInputElement.");
				return false;
			}
			return true;
		}

		public override void OnAddHandler(EventInterceptionArgs args)
		{
			base.OnAddHandler(args);
			(args.Instance as IInputElement).AddHandler(this.RoutedEvent, args.Handler);
		}

		public override void OnRemoveHandler(EventInterceptionArgs args)
		{
			base.OnRemoveHandler(args);
			(args.Instance as IInputElement).RemoveHandler(this.RoutedEvent, args.Handler);
		}

		public override void RuntimeInitialize(EventInfo eventInfo) =>
			this.RoutedEvent = EventManager.RegisterRoutedEvent(this.TargetName, RoutingStrategy.Bubble, this.EventHandlerType, this.DeclaringType);
	}
}
